@@ -42,6 +42,7 @@
 #include "gedit-utils.h"
 #include "gedit-message-area.h"
 #include "gedit-io-error-message-area.h"
+#include "gedit-print.h"
 #include "gedit-print-job-preview.h"
 #include "gedit-progress-message-area.h"
 #include "gedit-debug.h"
@@ -82,6 +83,8 @@ struct _GeditTabPrivate
         
 	gint	                not_editable : 1;
 	gint                    auto_save : 1;
+
+	gint                    ask_if_externally_modified : 1;
 };
 
 G_DEFINE_TYPE(GeditTab, gedit_tab, GTK_TYPE_VBOX)
@@ -166,7 +169,6 @@ remove_auto_save_timeout (GeditTab *tab)
 	tab->priv->auto_save_timeout = 0;
 }
 
-
 static void
 gedit_tab_get_property (GObject    *object,
 		        guint       prop_id,
@@ -293,7 +295,6 @@ gedit_tab_class_init (GeditTabClass *klass)
 	g_type_class_add_private (object_class, sizeof (GeditTabPrivate));
 }
 
-
 GeditTabState
 gedit_tab_get_state (GeditTab *tab)
 {
@@ -405,8 +406,6 @@ gedit_tab_set_state (GeditTab      *tab,
 	g_object_notify (G_OBJECT (tab), "state");		
 }
 
-
-
 static void 
 document_uri_notify_handler (GeditDocument *document,
 			     GParamSpec    *pspec,
@@ -461,17 +460,42 @@ remove_tab (GeditTab *tab)
 }
 
 static void 
-unrecoverable_loading_error_message_area_response (GeditMessageArea *message_area,
-						   gint              response_id,
-						   GeditTab         *tab)
+io_loading_error_message_area_response (GeditMessageArea *message_area,
+					gint              response_id,
+					GeditTab         *tab)
 {
-	remove_tab (tab);
+	if (response_id == GTK_RESPONSE_OK)
+	{
+		GeditDocument *doc;
+		gchar *uri;
+
+		doc = gedit_tab_get_document (tab);
+		g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+		uri = gedit_document_get_uri (doc);
+		g_return_if_fail (uri != NULL);
+
+		set_message_area (tab, NULL);
+		gedit_tab_set_state (tab, GEDIT_TAB_STATE_LOADING);
+
+		g_return_if_fail (tab->priv->auto_save_timeout <= 0);
+
+		gedit_document_load (doc,
+				     uri,
+				     tab->priv->tmp_encoding,
+				     tab->priv->tmp_line_pos,
+				     FALSE);
+	}
+	else
+	{
+		remove_tab (tab);
+	}
 }
 
 static void 
-recoverable_loading_error_message_area_response (GeditMessageArea *message_area,
-						 gint              response_id,
-						 GeditTab         *tab)
+conversion_loading_error_message_area_response (GeditMessageArea *message_area,
+						gint              response_id,
+						GeditTab         *tab)
 {
 	GeditDocument *doc;
 	gchar *uri;
@@ -485,17 +509,17 @@ recoverable_loading_error_message_area_response (GeditMessageArea *message_area,
 	if (response_id == GTK_RESPONSE_OK)
 	{
 		const GeditEncoding *encoding;
-		
+
 		encoding = gedit_conversion_error_message_area_get_encoding (
 				GTK_WIDGET (message_area));
 
 		g_return_if_fail (encoding != NULL);
-			
+
 		set_message_area (tab, NULL);
 		gedit_tab_set_state (tab, GEDIT_TAB_STATE_LOADING);
-	
+
 		tab->priv->tmp_encoding = encoding;
-		
+
 		g_return_if_fail (tab->priv->auto_save_timeout <= 0);
 
 		gedit_document_load (doc,
@@ -508,9 +532,7 @@ recoverable_loading_error_message_area_response (GeditMessageArea *message_area,
 	{
 		_gedit_recent_remove (GEDIT_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab))), uri);
 
-		unrecoverable_loading_error_message_area_response (message_area,
-								   response_id,
-								   tab);
+		remove_tab (tab);
 	}
 
 	g_free (uri);
@@ -924,11 +946,11 @@ document_loaded (GeditDocument *document,
 
 				if (tab->priv->state == GEDIT_TAB_STATE_LOADING_ERROR)
 				{
-					emsg = gedit_unrecoverable_loading_error_message_area_new (uri, 
-												   error);
+					emsg = gedit_io_loading_error_message_area_new (uri, 
+											error);
 					g_signal_connect (emsg,
 							  "response",
-							  G_CALLBACK (unrecoverable_loading_error_message_area_response),
+							  G_CALLBACK (io_loading_error_message_area_response),
 							  tab);
 				}
 				else
@@ -963,7 +985,7 @@ document_loaded (GeditDocument *document,
 
 			g_signal_connect (emsg,
 					  "response",
-					  G_CALLBACK (recoverable_loading_error_message_area_response),
+					  G_CALLBACK (conversion_loading_error_message_area_response),
 					  tab);
 		}
 
@@ -1039,6 +1061,8 @@ document_loaded (GeditDocument *document,
 		gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
 		
 		install_auto_save_timeout_if_needed (tab);
+
+		tab->priv->ask_if_externally_modified = TRUE;
 	}
 
  end:
@@ -1358,10 +1382,107 @@ document_saved (GeditDocument *document,
 			gedit_tab_set_state (tab, GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW);
 		else
 			gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
+
+		tab->priv->ask_if_externally_modified = TRUE;
 			
 		end_saving (tab);
 	}
+}
 
+static void 
+externally_modified_notification_message_area_response (GeditMessageArea *message_area,
+							gint              response_id,
+							GeditTab         *tab)
+{
+	GeditView *view;
+
+	set_message_area (tab, NULL);
+	view = gedit_tab_get_view (tab);
+
+	if (response_id == GTK_RESPONSE_OK)
+	{
+		_gedit_tab_revert (tab);
+	}
+	else
+	{
+		tab->priv->ask_if_externally_modified = FALSE;
+
+		/* go back to normal state */
+		gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
+	}
+
+	gtk_widget_grab_focus (GTK_WIDGET (view));
+}
+
+static void
+display_externally_modified_notification (GeditTab *tab)
+{
+	GtkWidget *message_area;
+	GeditDocument *doc;
+	gchar *uri;
+	gboolean document_modified;
+
+	doc = gedit_tab_get_document (tab);
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+	/* uri cannot be NULL, we're here because
+	 * the file we're editing changed on disk */
+	uri = gedit_document_get_uri (doc);
+	g_return_if_fail (uri != NULL);
+
+	document_modified = gtk_text_buffer_get_modified (GTK_TEXT_BUFFER(doc));
+	message_area = gedit_externally_modified_message_area_new (uri, document_modified);
+	g_free (uri);
+
+	tab->priv->message_area = NULL;
+	set_message_area (tab, message_area);
+	gtk_widget_show (message_area);
+
+	g_signal_connect (message_area,
+			  "response",
+			  G_CALLBACK (externally_modified_notification_message_area_response),
+			  tab);
+}
+
+static gboolean
+view_focused_in (GtkWidget     *widget,
+                 GdkEventFocus *event,
+                 GeditTab      *tab)
+{
+	GeditDocument *doc;
+
+	g_return_val_if_fail (GEDIT_IS_TAB (tab), FALSE);
+
+	/* we try to detect file changes only in the normal state */
+	if (tab->priv->state != GEDIT_TAB_STATE_NORMAL)
+	{
+		return FALSE;
+	}
+
+	/* we already asked, don't bug the user again */
+	if (!tab->priv->ask_if_externally_modified)
+	{
+		return FALSE;
+	}
+
+	doc = gedit_tab_get_document (tab);
+
+	/* If file was never saved or is remote we do not check */
+	if (!gedit_document_is_local (doc))
+	{
+		return FALSE;
+	}
+
+	if (_gedit_document_check_externally_modified (doc))
+	{
+		gedit_tab_set_state (tab, GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION);
+
+		display_externally_modified_notification (tab);
+
+		return FALSE;
+	}
+
+	return FALSE;
 }
 
 static void
@@ -1378,6 +1499,8 @@ gedit_tab_init (GeditTab *tab)
 	tab->priv->not_editable = FALSE;
 
 	tab->priv->save_flags = 0;
+
+	tab->priv->ask_if_externally_modified = TRUE;
 	
 	/* Create the scrolled window */
 	sw = gtk_scrolled_window_new (NULL, NULL);
@@ -1436,11 +1559,16 @@ gedit_tab_init (GeditTab *tab)
 			  "saved",
 			  G_CALLBACK (document_saved),
 			  tab);
-			  
-	g_signal_connect_after(tab->priv->view,
-			       "realize",
-			       G_CALLBACK (view_realized),
-			       tab);
+
+	g_signal_connect_after (tab->priv->view,
+				"focus-in-event",
+				G_CALLBACK (view_focused_in),
+				tab);
+
+	g_signal_connect_after (tab->priv->view,
+				"realize",
+				G_CALLBACK (view_realized),
+				tab);
 }
 
 GtkWidget *
@@ -1713,21 +1841,25 @@ _gedit_tab_get_icon (GeditTab *tab)
 						 GTK_STOCK_OPEN, 
 						 icon_size);
 			break;
+
 		case GEDIT_TAB_STATE_REVERTING:
 			pixbuf = get_stock_icon (theme, 
 						 GTK_STOCK_REVERT_TO_SAVED, 
 						 icon_size);						 
 			break;
+
 		case GEDIT_TAB_STATE_SAVING:
 			pixbuf = get_stock_icon (theme, 
 						 GTK_STOCK_SAVE, 
 						 icon_size);
 			break;
+
 		case GEDIT_TAB_STATE_PRINTING:
 			pixbuf = get_stock_icon (theme, 
 						 GTK_STOCK_PRINT, 
 						 icon_size);
 			break;
+
 		case GEDIT_TAB_STATE_PRINT_PREVIEWING:
 		case GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW:		
 			pixbuf = get_stock_icon (theme, 
@@ -1736,13 +1868,20 @@ _gedit_tab_get_icon (GeditTab *tab)
 			break;
 
 		case GEDIT_TAB_STATE_LOADING_ERROR:
-		case GEDIT_TAB_STATE_REVERTING_ERROR:		
+		case GEDIT_TAB_STATE_REVERTING_ERROR:
 		case GEDIT_TAB_STATE_SAVING_ERROR:
 		case GEDIT_TAB_STATE_GENERIC_ERROR:
 			pixbuf = get_stock_icon (theme, 
 						 GTK_STOCK_DIALOG_ERROR, 
 						 icon_size);
 			break;
+
+		case GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION:
+			pixbuf = get_stock_icon (theme,
+						 GTK_STOCK_DIALOG_WARNING,
+						 icon_size);
+			break;
+
 		default:
 		{
 			gchar *raw_uri;
@@ -1813,7 +1952,13 @@ _gedit_tab_revert (GeditTab *tab)
 	gchar *uri;
 
 	g_return_if_fail (GEDIT_IS_TAB (tab));
-	g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_NORMAL);
+	g_return_if_fail ((tab->priv->state == GEDIT_TAB_STATE_NORMAL) ||
+			  (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION));
+
+	if (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION)
+	{
+		set_message_area (tab, NULL);
+	}
 
 	doc = gedit_tab_get_document (tab);
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
@@ -1842,9 +1987,11 @@ void
 _gedit_tab_save (GeditTab *tab)
 {
 	GeditDocument *doc;
+	GeditDocumentSaveFlags save_flags;
 
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 	g_return_if_fail ((tab->priv->state == GEDIT_TAB_STATE_NORMAL) ||
+			  (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION) ||
 			  (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW));
 	g_return_if_fail (tab->priv->tmp_save_uri == NULL);
 	g_return_if_fail (tab->priv->tmp_encoding == NULL);
@@ -1853,6 +2000,21 @@ _gedit_tab_save (GeditTab *tab)
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 	g_return_if_fail (!gedit_document_is_untitled (doc));
 
+	if (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION)
+	{
+		/* We already told the user about the external
+		 * modification: hide the message area and set
+		 * the save flag.
+		 */
+
+		set_message_area (tab, NULL);
+		save_flags = tab->priv->save_flags | GEDIT_DOCUMENT_SAVE_IGNORE_MTIME;
+	}
+	else
+	{
+		save_flags = tab->priv->save_flags;
+	}
+
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_SAVING);
 
 	/* uri used in error messages, will be freed in document_saved */
@@ -1862,7 +2024,7 @@ _gedit_tab_save (GeditTab *tab)
 	if (tab->priv->auto_save_timeout > 0)
 		remove_auto_save_timeout (tab);
 		
-	gedit_document_save (doc, tab->priv->save_flags);
+	gedit_document_save (doc, save_flags);
 }
 
 static gboolean
@@ -1937,18 +2099,38 @@ _gedit_tab_save_as (GeditTab            *tab,
 		    const GeditEncoding *encoding)
 {
 	GeditDocument *doc;
+	GeditDocumentSaveFlags save_flags;
 
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 	g_return_if_fail ((tab->priv->state == GEDIT_TAB_STATE_NORMAL) ||
+			  (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION) ||
 			  (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW));
 	g_return_if_fail (encoding != NULL);
-				  
+
 	g_return_if_fail (tab->priv->tmp_save_uri == NULL);
 	g_return_if_fail (tab->priv->tmp_encoding == NULL);
-	
+
 	doc = gedit_tab_get_document (tab);
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 
+	/* reset the save flags, when saving as */
+	tab->priv->save_flags = 0;
+
+	if (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION)
+	{
+		/* We already told the user about the external
+		 * modification: hide the message area and set
+		 * the save flag.
+		 */
+
+		set_message_area (tab, NULL);
+		save_flags = tab->priv->save_flags | GEDIT_DOCUMENT_SAVE_IGNORE_MTIME;
+	}
+	else
+	{
+		save_flags = tab->priv->save_flags;
+	}
+
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_SAVING);
 
 	/* uri used in error messages... strdup because errors are async
@@ -1956,9 +2138,6 @@ _gedit_tab_save_as (GeditTab            *tab,
 	tab->priv->tmp_save_uri = g_strdup (uri);
 	tab->priv->tmp_encoding = encoding;
 
-	/* reset the save flags, when saving as */
-	tab->priv->save_flags = 0;
-	
 	if (tab->priv->auto_save_timeout > 0)
 		remove_auto_save_timeout (tab);
 
@@ -2142,12 +2321,15 @@ show_printing_message_area (GeditTab      *tab,
 
 void 
 _gedit_tab_print (GeditTab      *tab,
-		  GeditPrintJob *pjob,
+		  gpointer       print_job,
 		  GtkTextIter   *start, 
 		  GtkTextIter   *end)
 {
+	GeditPrintJob *pjob;
 	GeditDocument *doc;
-	
+
+	pjob = (GeditPrintJob *) print_job;
+
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 	g_return_if_fail (GEDIT_IS_PRINT_JOB (pjob));
 	g_return_if_fail (start != NULL);
@@ -2187,12 +2369,15 @@ _gedit_tab_print (GeditTab      *tab,
 
 void
 _gedit_tab_print_preview (GeditTab      *tab,
-			  GeditPrintJob *pjob,
+			  gpointer       print_job,
 			  GtkTextIter   *start, 
-			  GtkTextIter   *end)		  
+			  GtkTextIter   *end)
 {
+	GeditPrintJob *pjob;
 	GeditDocument *doc;
-	
+
+	pjob = (GeditPrintJob *) print_job;
+
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 	g_return_if_fail (GEDIT_IS_PRINT_JOB (pjob));
 	g_return_if_fail (start != NULL);
